<?php

namespace Drupal\nofraud;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Url;

/**
 * Provides a list controller for nofraud transaction entity.
 *
 * @ingroup nofraud
 */
class NoFraudListBuilder extends EntityListBuilder {

  /**
   * {@inheritdoc}
   */
  protected $limit = 10;

  /**
   * Loads entity IDs using a pager sorted by the entity id.
   *
   * @return array
   *   An array of entity IDs.
   */
  protected function getEntityIds() {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort($this->entityType->getKey('id'), 'DESC');

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }
    return $query->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build['description'] = [
      '#markup' => $this->t('Display the list of the available NoFraud transaction entries.'),
    ];

    $build += parent::render();
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('ID');
    $header['label'] = $this->t('Name');
    $header['order_id'] = $this->t('Order');
    $header['status'] = $this->t('Status');
    $header['canceled'] = $this->t('Canceled');
    $header['nofraud_id'] = $this->t('NoFraud ID');
    $header['user_id'] = $this->t('Author');
    $header['created'] = $this->t('Created');
    $header['changed'] = $this->t('Changed');

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function getOperations(EntityInterface $entity) {
    $operations = parent::getOperations($entity);
    $operations['view'] = [
      'title' => $this->t('View'),
      'url' => $this->ensureDestination($entity->toUrl('canonical')),
    ];
    $operations['cancel_transaction'] = [
      'title' => $this->t('Cancel Transaction'),
      'url' => $this->ensureDestination(Url::fromRoute('nofraud.cancel_transaction', ['nofraud' => $entity->id()])),
    ];
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\nofraud\Entity\NoFraudTransaction $entity */
    $row['id'] = $entity->id();
    $row['label'] = $entity->label();

    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $entity->order_id->entity;
    $row['order_id'] = $order ? $order->toLink(NULL, 'canonical')->toString() : $this->t('No order reference!');

    $row['status'] = $entity->status->value;
    $row['canceled'] = $entity->canceled->value ? $this->t('Yes') : $this->t('No');
    $row['nofraud_id'] = $entity->nofraud_id->value;
    $row['user_id'] = $entity->user_id->entity->toLink(NULL, 'canonical')->toString();
    $row['created'] = (new DrupalDateTime())->createFromTimestamp($entity->created->value)->format('Y-m-d H:i:s');
    $row['changed'] = (new DrupalDateTime())->createFromTimestamp($entity->changed->value)->format('Y-m-d H:i:s');

    return $row + parent::buildRow($entity);
  }

}
